/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>

#include "hexmap.h"

/**
 * Decode an url encoded string. This expects a valid url
 * encoded string and it size as arguments, else the behaviour
 * of this function is undefined.
 * This function modifies the data in buffer. No copy is made.
 * The reason for this is only performance.
 */
size_t
TR_urldecode(char * buffer, size_t nbuffer)
{
	char * buf_ptr = buffer;
	char * res_ptr = buffer;

	for(; 0 < nbuffer; nbuffer--, buf_ptr++, res_ptr++) {
		switch(*buf_ptr) {
			case '%':
				*res_ptr = HEXVAL(&(buf_ptr[1]));
				buf_ptr += 2;
				nbuffer -= 2;
				break;

			case '+':
				*buf_ptr = ' ';
				/* intended drop through */

			default:
				*res_ptr = *buf_ptr;
				break;
		}
	}

	return res_ptr - buffer;
}

// vim: set ft=c ts=4 sw=4:
